-- Useful - SCREEN:
local screen = {
    screenSource = Vector2 (guiGetScreenSize ()), -- Resolução do Cliente.
    screenBase = Vector2 (1366, 768), -- Resolução Base da Interface.
}

local sx, sy = (screen.screenSource['x'] / screen.screenBase['x']), (screen.screenSource['y'] / screen.screenBase['y'])

function respc (x, y, w, h)
    return (
        (x  / screen.screenBase['x']) * screen.screenSource['x']), 
        ((y / screen.screenBase['y']) * screen.screenSource['y']), 
        ((w / screen.screenBase['y']) * screen.screenSource['y']), 
        ((h / screen.screenBase['y']) * screen.screenSource['y']
    )
end

-- Useful - lerp
local effect = {}

function lerp (a, b, t) 
    return (a * (1 - t) + b * t)
end

function interpolate (from, to, time, index)
    if not effect[index] then
        effect[index] = from
    end
    effect[index] = lerp (effect[index], to, time)
    return math.ceil (effect[index])
end

-- SVG:
svg = {
    home = svgCreate (304, 336, [[<svg width="304" height="336" viewBox="0 0 304 336" fill="none" xmlns="http://www.w3.org/2000/svg">
    <rect width="304" height="336" rx="8" fill="#161616"/>
    <path d="M23 108H282V277C282 279.761 279.761 282 277 282H28C25.2386 282 23 279.761 23 277V108Z" fill="#1C1C1C"/>
    <rect x="23" y="64" width="259" height="34" rx="5" fill="#1C1C1C"/>
    </svg>
    ]]),
    submit = svgCreate (125, 33, [[<svg width="125" height="33" viewBox="0 0 125 33" fill="none" xmlns="http://www.w3.org/2000/svg">
    <rect width="125" height="33" rx="5" fill="white"/>
    </svg>
    ]]),
}

-- dx:functions
local textures = {
    image = {},
    font = {},
    fade = {}
}

_dxDrawImage = dxDrawImage
function dxDrawImage (x, y, w, h, path, ...)
    local x, y, w, h = respc (x, y, w, h)
    if path == 'string' then
        if not textures.image[path] then
            textures.image[path] = dxCreateTexture (
                tostring (path),
                'dxt5',
                false,
                'clamp'
            )
        end
        path = textures.image[path]
    end
    return _dxDrawImage (x, y, w, h, path, ...)
end

_dxDrawRectangle = dxDrawRectangle
function dxDrawRectangle (x, y, w, h, ...)
    local x, y, w, h = respc (x, y, w, h)
    return _dxDrawRectangle (x, y, w, h, ...)
end

_dxCreateFont = dxCreateFont
function dxCreateFont (path, size, ...)
    if not textures.font[path] then
        textures.font[path] = {}
    end
    if not textures.font[path][size] then
        textures.font[path][size] = _dxCreateFont (
            tostring (path),
            (sy * size),
            ...
        )
    end
    return textures.font[path][size]
end

_dxDrawText = dxDrawText
function dxDrawText (text, x, y, w, h, ...)
    local x, y, w, h = respc (x, y, w, h)
    return _dxDrawText (tostring (text), x, y, (x + w), (y + h), ...)
end

_getCursorPosition = getCursorPosition
function getCursorPosition ()
    local mouse = Vector2 (_getCursorPosition ())
    return mouse['x'] * screen.screenSource['x'], mouse['y'] * screen.screenSource['y']
end

function isCursorOnElement (x, y, w, h)
    local cursor = Vector2 (getCursorPosition ())
    return (cursor['x'] > x and cursor['x'] < x + w and cursor['y'] > y and cursor['y'] < y + h)
end

_isCursorOnElement = isCursorOnElement
function isCursorOnElement (x, y, w, h)
    local x, y, w, h = respc (x, y, w, h)
    return _isCursorOnElement (x, y, w, h)
end

-- Lib - Editbox:
local editbox = {};

function dxCreateEditbox(Id, Position, MaxCharacters, Color, Font, Placeholder, hasLine, isPassword, isNumber, hasBlock) -- Função para criar a editbox.
    if (not editbox[Id]) then
        editbox[Id] = {
            Position = Position;
            MaxCharacters = MaxCharacters;
            Color = Color;
            Font = (Font or "default");
            Placeholder = Placeholder;
            Text = "";
            isPassword = isPassword;
            isNumber = isNumber;
            hasLine = hasLine;
            hasSelect = false;
            hasPressed = nil;
            hasBlock = hasBlock;
        }
    end
    local values = editbox[Id];
    values['Color'] = Color
    values['Position'] = Position
    local x, y, w, h = unpack(values["Position"]);
    if (values["Color"][2]) then
        dxDrawRectangle(x, y, w, h, values["Color"][2]);
    end
    local text = (values["isPassword"] and #values["Text"] > 0 and string.gsub(values["Text"], ".", "•") or (values["hasSelect"] and values["Text"] and values["Text"] or (#values["Text"] > 0 and values["Text"] or values["Placeholder"])));
    dxDrawText(text, x, y, w, h, values["Color"][1], 1.00, values["Font"], "center", "center", true);
    if (values["hasLine"]) then
        if (values["hasSelect"] and #values["Text"] > 0) then
            local width, heigth = dxGetTextSize(text, 0, 1.0, 1.0, values["Font"], false, false);
            dxDrawLine(x + (width) + (10), y + (heigth) - (3), x + (width) + (10), y + (heigth*2), tocolor(255, 255, 255, math.abs(math.sin(getTickCount()/800) * 255)), 1, false);
        end
    end
    if (values["hasPressed"] and getTickCount() >= values["hasPressed"] + 500) then
        if ((#values["Text"] - 1) >= 0) then
            values["Text"] = string.sub(values["Text"], 1, (#values["Text"] - 1));
        end
    end
end

-- Click:
addEventHandler("onClientClick", getRootElement(), function(btn, state)
    if (not isCursorShowing()) then
        return false;
    end
    if (btn == "left" and state == "down") then
        dxDisableSelects();
        for Id, Value in pairs(editbox) do
            if (Value["hasBlock"]) then return end;
            if (isCursorOnElement(unpack(Value["Position"]))) then
                guiSetInputMode("no_binds");
                Value["hasSelect"] = true;
                return true;
            end
        end
    end
end)

-- Digitar:
addEventHandler("onClientCharacter", getRootElement(), function(character)
    if (not isCursorShowing()) then
        return false;
    end
    if (character) then
        for Id, Value in pairs(editbox) do
            if (Value["hasSelect"]) then
                if (#Value["Text"] + 1 <= Value["MaxCharacters"]) then
                    if (Value["isNumber"]) then
                        if (tonumber(character)) then
                            Value["Text"] = Value["Text"]..character;
                        end
                    else
                        Value["Text"] = Value["Text"]..character;    
                    end
                end
                break
            end
        end
    end
end)

-- Disable Selects:
function dxDisableSelects()
    for Id, Value in pairs(editbox) do
        Value["hasSelect"] = false;
        Value["hasPressed"] = nil;
    end
    guiSetInputMode("no_binds_when_editing");
    return true;
end

-- Backspace:
addEventHandler("onClientKey", getRootElement(), function(key, press)
    if (key == "backspace") then
        for Id, Value in pairs(editbox) do
            if (Value["hasSelect"]) then
                if (press) then
                    if ((#Value["Text"] - 1) >= 0) then
                        Value["Text"] = string.sub(Value["Text"], 1, (#Value["Text"] - 1));
                        Value["hasPressed"] = getTickCount();
                    end
                else
                    Value["hasPressed"] = false;
                end
            end
        end
    end
end)

-- Editbox Functions:
function dxEditboxSetText(identify, newText) -- Função para alterar o texto da editbox.
    if (not identify or not editbox[identify]) then
        return false
    end
    if (not newText or type(newText) ~= "string" or #newText == 0) then
        return false
    end
    editbox[identify]["Text"] = newText;
    return true;
end

function dxEditboxGetText(identify) -- Função para obter o texto da editbox.
    if (not identify or not editbox[identify]) then
        return
    end
    return {content = (editbox[identify]["isNumber"] and tonumber(editbox[identify]["Text"]) or tostring(editbox[identify]["Text"])), type = type(editbox[identify]["Text"]), len = string.len(tostring(editbox[identify]["Text"]))};
end

function dxEditShowPassword(identify, toggle) -- Função para manipular a visualização da opção "password" da editbox.
    if (not identify or not editbox[identify]) then
        return false
    end
    if (not toggle or type(toggle) ~= "boolean") then
        return false
    end
    editbox[identify]["isPassword"] = toggle;
    return true;
end

function dxEditGetPasswordState(identify) -- Função para obter o valor "boolean" da opção "password" da editbox.
    if (not identify or not editbox[identify]) then
        return false
    end
    return editbox[identify]["isPassword"];
end

function dxEditboxForceSelect(identify) -- Função para forçar o client a selecionar a editbox prédefinida.
    if (not identify or not editbox[identify]) then
        return false
    end
    dxDisableSelects();
    guiSetInputMode("no_binds");
    editbox[identify]["hasSelect"] = true;
    return true;
end

function dxEditboxDestroy(identify) -- Função para destroir a editbox.
    if (not identify or not editbox[identify]) then
        return false
    end
    dxDisableSelects();
    editbox[identify] = nil;
    return true;
end

function dxEditboxSetBlock(identify, toggle) -- Função para manipular o bloqueio da editbox.
    if (not identify or not editbox[identify]) then
        return false
    end
    if (not toggle or type(toggle) ~= "boolean") then
        return false
    end
    dxDisableSelects();
    editbox[identify]["hasBlock"] = toggle;
    return true;
end

function dxEditboxGetBlock(identify) -- Função para obter o valor "boolean" da opção "hasBlock" da editbox.
    if (not identify or not editbox[identify]) then
        return false
    end
    return editbox[identify]["hasBlock"];
end

function dxEditboxDestroyAll ()
    for i, v in pairs (editbox) do
        dxEditboxDestroy(i)
    end
end